import zmq
import logging

class ZMQTestSubscriber:
    """
    A class to subscribe to a ZeroMQ message bus and log received messages.
    """

    def __init__(self, port="5556", tokens=None):
        """
        Initializes the ZMQTestSubscriber with specified port and tokens.
        :param port: Port number to connect to for the ZeroMQ message bus.
        :param tokens: List of tokens to subscribe to. If None or empty, subscribes to all messages.
        """
        self.port = port
        self.tokens = tokens
        self.logger = self.setup_logger()
        self.subscribed = False # indicates subscriber subscription status

    def setup_logger(self):
        """
        Sets up the logger for the class.
        Configures logging to output both to the console and to a file.
        """
        logger = logging.getLogger("MessageBusTester")
        logger.setLevel(logging.INFO)

        c_handler = logging.StreamHandler()
        f_handler = logging.FileHandler('message_bus_test.log')

        c_format = logging.Formatter('%(name)s - %(levelname)s - %(message)s')
        f_format = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        c_handler.setFormatter(c_format)
        f_handler.setFormatter(f_format)

        logger.addHandler(c_handler)
        logger.addHandler(f_handler)

        return logger

    def subscribe(self):
        """
        Subscribes to the ZeroMQ message bus and logs received messages.
        """
        context = zmq.Context()
        socket = context.socket(zmq.SUB)
        socket.connect(f"tcp://localhost:{self.port}")

        if self.tokens is None or not self.tokens:
            socket.setsockopt_string(zmq.SUBSCRIBE, "")
            self.logger.info("Subscribed to all messages.")
        else:
            for token in self.tokens:
                socket.setsockopt_string(zmq.SUBSCRIBE, token)
            self.logger.info(f"Subscribed to tokens: {self.tokens}")

        self.subscribed = True
        self.logger.info(f"Listening for messages on port {self.port}...")
        while True:
            try:
                message = socket.recv_string()
                token, data = message.split(' & ')
                if self.tokens is None or token in self.tokens:
                    self.logger.info(f"Received message for token {token}: {data}")
            except zmq.ZMQError as e:
                self.logger.error(f"ZMQ Error: {e}")
                self.subscribed = False
                break
        
    def is_subscribed(self):
        """
        Checks if the subscriber is currently subscribed
        """

        return self.subscribed



import asyncio
import websockets
import json
import logging
from typing import List

# Configure logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

class AuthenticatedWebsocketConnector:
    def __init__(self, uri, jwt_token, ping_interval=10):
        self.uri = uri
        self.jwt_token = jwt_token
        self.ping_interval = ping_interval
        self.websocket = None

    async def connect(self):
        try:
            self.websocket = await websockets.connect(self.uri, extra_headers={"Authorization": f"Bearer {self.jwt_token}"})
            asyncio.create_task(self.ping_pong())
        except Exception as e:
            logger.error(f"Error during connection: {e}")
            raise

    async def send_message(self, action, tokens):
        if self.websocket:
            try:
                message = f"{action}|{','.join(tokens)}"
                await self.websocket.send(message)
            except Exception as e:
                logger.error(f"Error sending message: {e}")
                raise

    async def receive_message(self):
        if self.websocket:
            try:
                return await self.websocket.recv()
            except Exception as e:
                logger.error(f"Error receiving message: {e}")
                raise

    async def ping_pong(self):
        while self.websocket:
            try:
                await self.websocket.ping()
                await asyncio.sleep(self.ping_interval)
            except websockets.exceptions.ConnectionClosed as e:
                logger.error(f"Connection closed: {e}")
                break
            except Exception as e:
                logger.error(f"Error in ping_pong: {e}")
                break

    async def close(self):
        if self.websocket:
            try:
                await self.websocket.close()
            except Exception as e:
                logger.error(f"Error closing websocket: {e}")
                raise


import requests

class AuthClient:
    def __init__(self, base_url="http://localhost:8000"):
        self.base_url = base_url
        self.token_file = "auth_token.txt"

    def register_user(self, username, password):
        """Function to register a new user."""
        try:
            response = requests.post(f"{self.base_url}/api/v1/users/register", json={"username": username, "password": password})
            if response.status_code == 200:
                return "Registration successful."
            else:
                return f"Registration failed: {response.json().get('detail', 'Unknown error')}"
        except requests.ConnectionError:
            return "Error: Unable to connect to the server."
        except requests.Timeout:
            return "Error: Server timed out."
        except requests.RequestException as e:
            return f"Error: {e}"

    def login_user(self, username, password):
        """Function to log in an existing user."""
        try:
            response = requests.post(f"{self.base_url}/api/v1/users/login", json={"username": username, "password": password})
            if response.status_code == 200:
                response_data = response.json()
                token = response_data.get("access_token")
                token = token['access_token']
                self.write_token_to_file(str(token))
                return {
                    "message": "Login successful.",
                    "username": username,
                    "access_token": token,
                    "token_type": response_data.get("token_type")
                }
            else:
                return f"Login failed: {response.json().get('detail', 'Unknown error')}"
        except requests.ConnectionError:
            return "Error: Unable to connect to the server."
        except requests.Timeout:
            return "Error: Server timed out."
        except requests.RequestException as e:
            return f"Error: {e}"

    def write_token_to_file(self, token):
        """Write the JWT token to a file."""
        try:
            with open(self.token_file, 'w') as file:
                file.write(token)
            return f"Token written to {self.token_file}"
        except IOError as e:
            return f"Error writing token to file: {e}"



import unittest
import os
import sys

# Adjust the path to include the parent directory so we can import the library
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from aziraclient.message_bus.message_bus_tester import ZMQTestSubscriber

class TestZMQTestSubscriber(unittest.TestCase):
    """
    Test cases for ZMQTestSubscriber class.
    """

    def test_subscribe(self):
        """
        Test the subscribe functionality of the ZMQTestSubscriber.
        """
        # Initialize the ZMQTestSubscriber with specific tokens
        tokens = ["NSE:26009", "NSE:26000"]
        subscriber = ZMQTestSubscriber(tokens=tokens)

        # Calls the subscribe method
        # subscriber.subscribe()


        #Checks if the subscriber is correctly set up
        self.assertTrue(subscriber.is_subscribed)

        # Note: Actual testing of ZMQ subscriptions might require a running ZMQ server
        # and could be more complex than a simple unit test.

if __name__ == '__main__':
    unittest.main()



import unittest
import sys
import os

# Adjust the path to import the library correctly
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from aziraclient.auth.auth_client import AuthClient

class TestAuthClient(unittest.TestCase):
    """
    Test cases for the AuthClient class.
    """

    @classmethod
    def setUpClass(cls):
        """
        Set up that runs once before all tests.
        """
        cls.base_url = "http://localhost:8000"
        cls.token_file = "user_token.txt"
        cls.auth_client = AuthClient(base_url=cls.base_url)

    def test_register_user(self):
        """
        Test the register_user method.
        """
        response = self.auth_client.register_user("username1", "password")
        # TODO: Need to properly work on the assertion
        self.assertIn("Registration successful", response)

    def test_login_user(self):
        """
        Test the login_user method.
        """
        response = self.auth_client.login_user("username1", "password")
        self.assertIn("Login successful", response)

if __name__ == '__main__':
    unittest.main()



import unittest
import asyncio
import sys
import os
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from aziraclient.subscription.subscription import SubscribeToToken

class TestSubscriber(unittest.TestCase):
    """
    Test cases for ZMQTestSubscriber class.
    """

    def test_subscribe(self):
        """
        Test the subscribe functionality of the ZMQTestSubscriber.
        """
        # Initialize the ZMQTestSubscriber with specific tokens
        username = "<Enter your username>"
        password = "<Enter your password>"
        jwt_token = "<Enter your generated jwt token>"
        action = "subscribe"
        token = "NSE:26009"
        tester = SubscribeToToken(username, jwt_token, action, token)
        # asyncio.get_event_loop().run_until_complete(tester.test_connection())


        #Checks if the subscriber is correctly set up
        self.assertTrue(tester.is_subscribed)

        # Note: Actual testing of ZMQ subscriptions might require a running ZMQ server
        # and could be more complex than a simple unit test.

if __name__ == '__main__':
    unittest.main()



import asyncio
import websockets
import logging
import sys
import os
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from connector.connection import AuthenticatedWebsocketConnector

class SubscribeToToken:
    """
    A class to test WebSocket connections and message handling.
    """

    def __init__(self, username, jwt_token, action, token):
        """
        Initializes the SubscribeToToken with user credentials and action details.
        :param username: Username for the WebSocket connection.
        :param jwt_token: JWT token for authentication.
        :param action: Action to perform (e.g., 'subscribe' or 'unsubscribe').
        :param token: The token to subscribe or unsubscribe.
        """
        self.username = username
        self.jwt_token = jwt_token
        self.action = action
        self.token = token
        self.uri = f"ws://localhost:8000/api/trigger/ws/{username}"
        self.connector = AuthenticatedWebsocketConnector(self.uri, jwt_token)
        self.logger = self.setup_logger()
        self.subscribed = False

    def setup_logger(self):
        """
        Sets up the logger for the class.
        """
        logger = logging.getLogger("SubscribeToToken")
        logger.setLevel(logging.INFO)
        handler = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
        handler.setFormatter(formatter)
        logger.addHandler(handler)
        return logger

    async def test_connection(self):
        """
        Tests the WebSocket connection by sending and receiving messages.
        """
        await self.connector.connect()
        await self.connector.send_message(self.action, [self.token])
        try:
            confirmation = await self.connector.receive_message()
            self.logger.info(f"Confirmation of action received: {confirmation}")
            self.subscribed = True

            while True:
                message = await self.connector.receive_message()
                self.logger.info(f"Received message: {message}")

        except websockets.exceptions.ConnectionClosed as e:
            self.logger.error(f"Connection closed")
            self.subscribed = False
        finally:
            await self.connector.close()

    def is_subscribed(self):
        """
        Checks if the subscriber is currently subscribed
        """

        return self.subscribed




from setuptools import setup, find_packages

setup(
    name='aziraclient',
    version='0.1.3',
    author='Emmanuel Akanji',
    author_email='omoebun52@gmail.com',
    description='A library built for azira application to make it easier for users to subscribe to crypto tokens and continuously receive data in near real-time',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/manny-uncharted/aziraclient',  # Replace with your own GitHub URL
    packages=find_packages(),
    install_requires=[
        "amqp==5.2.0",
        "annotated-types==0.6.0",
        "anyio==3.7.1",
        "async-timeout==4.0.3",
        "fastapi==0.104.1",
        "fastapi-socketio==0.0.10",
        "httpcore==1.0.1",
        "httpx==0.25.1",
        "idna==3.4",
        "iniconfig==2.0.0",
        "iso8601==2.1.0",
        "jwt==1.3.1",
        "kombu==5.3.3",
        "Mako==1.3.0",
        "pytest==7.4.3",
        "python-engineio==4.8.0",
        "python-socketio==5.10.0",
        "requests==2.31.0",
        "websockets==12.0",
        "zmq==0.0.0",

    ],
    classifiers=[
        # Classifiers help users find your project
        # Full list: https://pypi.org/classifiers/
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)
